@@ -35,6 +35,7 @@
 
 #include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
+#include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-names.h>
 
 #define DEBUG_FLAG NAUTILUS_DEBUG_BOOKMARKS
@@ -64,12 +65,15 @@ struct NautilusBookmarkDetails
 	char *name;
 	gboolean has_custom_name;
 	GFile *location;
-	gboolean location_gone;
 	GIcon *icon;
 	GIcon *symbolic_icon;
 	NautilusFile *file;
 	
 	char *scroll_file;
+
+	gboolean exists;
+	guint exists_id;
+	GCancellable *cancellable;
 };
 
 static void	  nautilus_bookmark_disconnect_file	  (NautilusBookmark	 *file);
@@ -151,10 +155,8 @@ bookmark_file_changed_callback (NautilusFile *file,
 		 * bookmark just because its not there anymore.
 		 */
 		DEBUG ("%s: trashed", nautilus_bookmark_get_name (bookmark));
-		bookmark->details->location_gone = TRUE;
 		nautilus_bookmark_disconnect_file (bookmark);
 	} else {
-		bookmark->details->location_gone = FALSE;
 		bookmark_set_name_from_ready_file (bookmark, file);
 	}
 }
@@ -182,28 +184,78 @@ apply_warning_emblem (GIcon **base,
 	*base = emblemed_icon;
 }
 
+gboolean
+nautilus_bookmark_get_is_builtin (NautilusBookmark *bookmark)
+{
+	GUserDirectory xdg_type;
+
+	/* if this is not an XDG dir, it's never builtin */
+	if (!nautilus_bookmark_get_xdg_type (bookmark, &xdg_type)) {
+		return FALSE;
+	}
+
+	/* exclude XDG locations which are not in our builtin list */
+	if (xdg_type == G_USER_DIRECTORY_DESKTOP &&
+	    !g_settings_get_boolean (gnome_background_preferences, NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
+		return FALSE;
+	}
+
+	return (xdg_type != G_USER_DIRECTORY_TEMPLATES) && (xdg_type != G_USER_DIRECTORY_PUBLIC_SHARE);
+}
+
+gboolean
+nautilus_bookmark_get_xdg_type (NautilusBookmark *bookmark,
+				GUserDirectory   *directory)
+{
+	gboolean match;
+	GFile *location;
+	const gchar *path;
+	GUserDirectory dir;
+
+	match = FALSE;
+
+	for (dir = 0; dir < G_USER_N_DIRECTORIES; dir++) {
+		path = g_get_user_special_dir (dir);
+		if (!path) {
+			continue;
+		}
+
+		location = g_file_new_for_path (path);
+		match = g_file_equal (location, bookmark->details->location);
+		g_object_unref (location);
+
+		if (match) {
+			break;
+		}
+	}
+
+	if (match && directory != NULL) {
+		*directory = dir;
+	}
+
+	return match;
+}
+
 static GIcon *
 get_native_icon (NautilusBookmark *bookmark,
 		 gboolean symbolic)
 {
-	gint idx;
+	GUserDirectory xdg_type;
 	GIcon *icon = NULL;
 
 	if (bookmark->details->file == NULL) {
 		goto out;
 	}
 
-	for (idx = 0; idx < G_USER_N_DIRECTORIES; idx++) {
-		if (nautilus_file_is_user_special_directory (bookmark->details->file, idx)) {
-			break;
-		}
+	if (!nautilus_bookmark_get_xdg_type (bookmark, &xdg_type)) {
+		goto out;
 	}
 
-	if (idx < G_USER_N_DIRECTORIES) {
+	if (xdg_type < G_USER_N_DIRECTORIES) {
 		if (symbolic) {
-			icon = nautilus_special_directory_get_symbolic_icon (idx);
+			icon = nautilus_special_directory_get_symbolic_icon (xdg_type);
 		} else {
-			icon = nautilus_special_directory_get_icon (idx);
+			icon = nautilus_special_directory_get_icon (xdg_type);
 		}
 	}
 
@@ -240,7 +292,7 @@ nautilus_bookmark_set_icon_to_default (NautilusBookmark *bookmark)
 		g_free (uri);
 	}
 
-	if (nautilus_bookmark_uri_known_not_to_exist (bookmark)) {
+	if (!bookmark->details->exists) {
 		DEBUG ("%s: file does not exist, add emblem", nautilus_bookmark_get_name (bookmark));
 
 		apply_warning_emblem (&icon, FALSE);
@@ -270,6 +322,16 @@ nautilus_bookmark_disconnect_file (NautilusBookmark *bookmark)
 						      bookmark);
 		g_clear_object (&bookmark->details->file);
 	}
+
+	if (bookmark->details->cancellable != NULL) {
+		g_cancellable_cancel (bookmark->details->cancellable);
+		g_clear_object (&bookmark->details->cancellable);
+	}
+
+	if (bookmark->details->exists_id != 0) {
+		g_source_remove (bookmark->details->exists_id);
+		bookmark->details->exists_id = 0;
+	}
 }
 
 static void
@@ -281,7 +343,7 @@ nautilus_bookmark_connect_file (NautilusBookmark *bookmark)
 		return;
 	}
 
-	if (!nautilus_bookmark_uri_known_not_to_exist (bookmark)) {
+	if (bookmark->details->exists) {
 		DEBUG ("%s: creating file", nautilus_bookmark_get_name (bookmark));
 
 		bookmark->details->file = nautilus_file_get (bookmark->details->location);
@@ -306,6 +368,83 @@ nautilus_bookmark_connect_file (NautilusBookmark *bookmark)
 	}
 }
 
+static void
+nautilus_bookmark_set_exists (NautilusBookmark *bookmark,
+			      gboolean exists)
+{
+	if (bookmark->details->exists == exists) {
+		return;
+	}
+
+	bookmark->details->exists = exists;
+	DEBUG ("%s: setting bookmark to exist: %d\n",
+	       nautilus_bookmark_get_name (bookmark), exists);
+
+	/* refresh icon */
+	nautilus_bookmark_set_icon_to_default (bookmark);
+}
+
+static gboolean
+exists_non_native_idle_cb (gpointer user_data)
+{
+	NautilusBookmark *bookmark = user_data;
+	nautilus_bookmark_set_exists (bookmark, FALSE);
+
+	return FALSE;
+}
+
+static void
+exists_query_info_ready_cb (GObject *source,
+			    GAsyncResult *res,
+			    gpointer user_data)
+{
+	GFileInfo *info;
+	NautilusBookmark *bookmark;
+	GError *error = NULL;
+	gboolean exists = FALSE;
+
+	info = g_file_query_info_finish (G_FILE (source), res, &error);
+	if (!info && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		g_clear_error (&error);
+		return;
+	}
+
+	g_clear_error (&error);
+	bookmark = user_data;
+
+	if (info) {
+		exists = TRUE;
+
+		g_object_unref (info);
+		g_clear_object (&bookmark->details->cancellable);
+	}
+
+	nautilus_bookmark_set_exists (bookmark, exists);
+}
+
+static void
+nautilus_bookmark_update_exists (NautilusBookmark *bookmark)
+{
+	/* Convert to a path, returning FALSE if not local. */
+	if (!g_file_is_native (bookmark->details->location) &&
+	    bookmark->details->exists_id == 0) {
+		bookmark->details->exists_id =
+			g_idle_add (exists_non_native_idle_cb, bookmark);
+		return;
+	}
+
+	if (bookmark->details->cancellable != NULL) {
+		return;
+	}
+
+	bookmark->details->cancellable = g_cancellable_new ();
+	g_file_query_info_async (bookmark->details->location,
+				 G_FILE_ATTRIBUTE_STANDARD_TYPE,
+				 0, G_PRIORITY_DEFAULT,
+				 bookmark->details->cancellable,
+				 exists_query_info_ready_cb, bookmark);
+}
+
 /* GObject methods */
 
 static void
@@ -408,6 +547,7 @@ nautilus_bookmark_constructed (GObject *obj)
 	NautilusBookmark *self = NAUTILUS_BOOKMARK (obj);
 
 	nautilus_bookmark_connect_file (self);
+	nautilus_bookmark_update_exists (self);
 }
 
 static void
@@ -474,6 +614,8 @@ nautilus_bookmark_init (NautilusBookmark *bookmark)
 {
 	bookmark->details = G_TYPE_INSTANCE_GET_PRIVATE (bookmark, NAUTILUS_TYPE_BOOKMARK,
 							 NautilusBookmarkDetails);
+
+	bookmark->details->exists = TRUE;
 }
 
 const gchar *
@@ -549,52 +691,6 @@ nautilus_bookmark_compare_with (gconstpointer a, gconstpointer b)
 	return 0;
 }
 
-/**
- * nautilus_bookmark_compare_uris:
- *
- * Check whether the uris of two bookmarks are for the same location.
- * @a: first NautilusBookmark*.
- * @b: second NautilusBookmark*.
- * 
- * Return value: 0 if @a and @b have matching uri, 1 otherwise 
- * (GCompareFunc style)
- **/
-int		    
-nautilus_bookmark_compare_uris (gconstpointer a, gconstpointer b)
-{
-	NautilusBookmark *bookmark_a;
-	NautilusBookmark *bookmark_b;
-
-	g_return_val_if_fail (NAUTILUS_IS_BOOKMARK (a), 1);
-	g_return_val_if_fail (NAUTILUS_IS_BOOKMARK (b), 1);
-
-	bookmark_a = NAUTILUS_BOOKMARK (a);
-	bookmark_b = NAUTILUS_BOOKMARK (b);
-
-	return !g_file_equal (bookmark_a->details->location,
-			      bookmark_b->details->location);
-}
-
-NautilusBookmark *
-nautilus_bookmark_copy (NautilusBookmark *bookmark)
-{
-	NautilusBookmark *retval;
-
-	g_return_val_if_fail (NAUTILUS_IS_BOOKMARK (bookmark), NULL);
-
-	retval = nautilus_bookmark_new (bookmark->details->location,
-					bookmark->details->has_custom_name ?
-					bookmark->details->name : NULL);
-	if (bookmark->details->icon) {
-		g_object_set (retval, "icon", bookmark->details->icon, NULL);
-	}
-	if (bookmark->details->symbolic_icon) {
-		g_object_set (retval, "symbolic-icon", bookmark->details->symbolic_icon, NULL);
-	}
-
-	return retval;
-}
-
 GIcon *
 nautilus_bookmark_get_symbolic_icon (NautilusBookmark *bookmark)
 {
@@ -711,29 +807,6 @@ nautilus_bookmark_menu_item_new (NautilusBookmark *bookmark)
 	return menu_item;
 }
 
-gboolean
-nautilus_bookmark_uri_known_not_to_exist (NautilusBookmark *bookmark)
-{
-	char *path_name;
-	gboolean exists;
-
-	if (bookmark->details->location_gone) {
-		return TRUE;
-	}
-
-	/* Convert to a path, returning FALSE if not local. */
-	if (!g_file_is_native (bookmark->details->location)) {
-		return FALSE;
-	}
-	path_name = g_file_get_path (bookmark->details->location);
-
-	/* Now check if the file exists (sync. call OK because it is local). */
-	exists = g_file_test (path_name, G_FILE_TEST_EXISTS);
-	g_free (path_name);
-
-	return !exists;
-}
-
 void
 nautilus_bookmark_set_scroll_pos (NautilusBookmark      *bookmark,
 				  const char            *uri)
@@ -747,3 +820,9 @@ nautilus_bookmark_get_scroll_pos (NautilusBookmark      *bookmark)
 {
 	return g_strdup (bookmark->details->scroll_file);
 }
+
+gboolean
+nautilus_bookmark_get_exists (NautilusBookmark *bookmark)
+{
+	return bookmark->details->exists;
+}
