@@ -56,10 +56,12 @@ struct _NautilusTreeViewDragDestDetails {
 	GtkSelectionData *drag_data;
 	GList *drag_list;
 
+	guint hover_id;
 	guint highlight_id;
 	guint scroll_id;
 
 	char *direct_save_uri;
+	char *target_uri;
 };
 
 enum {
@@ -70,6 +72,7 @@ enum {
 	HANDLE_URI_LIST,
 	HANDLE_TEXT,
 	HANDLE_RAW,
+	HANDLE_HOVER,
 	LAST_SIGNAL
 };
 
@@ -247,6 +250,15 @@ get_drag_data (NautilusTreeViewDragDest *dest,
 	return TRUE;
 }
 
+static void
+remove_hover_timer (NautilusTreeViewDragDest *dest)
+{
+	if (dest->details->hover_id != 0) {
+		g_source_remove (dest->details->hover_id);
+		dest->details->hover_id = 0;
+	}
+}
+
 static void
 free_drag_data (NautilusTreeViewDragDest *dest)
 {
@@ -264,6 +276,50 @@ free_drag_data (NautilusTreeViewDragDest *dest)
 
 	g_free (dest->details->direct_save_uri);
 	dest->details->direct_save_uri = NULL;
+
+	g_free (dest->details->target_uri);
+	dest->details->target_uri = NULL;
+
+	remove_hover_timer (dest);
+}
+
+static gboolean
+hover_timer (gpointer user_data)
+{
+	NautilusTreeViewDragDest *dest = user_data;
+
+	dest->details->hover_id = 0;
+
+	g_signal_emit (dest, signals[HANDLE_HOVER], 0, dest->details->target_uri);
+
+	return FALSE;
+}
+
+static void
+check_hover_timer (NautilusTreeViewDragDest *dest,
+		   const char               *uri)
+{
+	GtkSettings *settings;
+	guint timeout;
+
+	if (g_strcmp0 (uri, dest->details->target_uri) == 0) {
+		return;
+	}
+	remove_hover_timer (dest);
+
+	settings = gtk_widget_get_settings (GTK_WIDGET (dest->details->tree_view));
+	g_object_get (settings, "gtk-timeout-expand", &timeout, NULL);
+
+	g_free (dest->details->target_uri);
+	dest->details->target_uri = NULL;
+
+	if (uri != NULL) {
+		dest->details->target_uri = g_strdup (uri);
+		dest->details->hover_id =
+			gdk_threads_add_timeout (timeout,
+						 hover_timer,
+						 dest);
+	}
 }
 
 static char *
@@ -453,11 +509,16 @@ drag_motion_callback (GtkWidget *widget,
 					 NULL);
 	
 	if (action) {
+		char *uri;
 		set_drag_dest_row (dest, drop_path);
+		uri = get_drop_target_uri_for_path (dest, path);
+		check_hover_timer (dest, uri);
+		g_free (uri);
 	} else {
 		clear_drag_dest_row (dest);
+		remove_hover_timer (dest);
 	}
-	
+
 	if (path) {
 		gtk_tree_path_free (path);
 	}
@@ -805,7 +866,7 @@ drag_data_received_callback (GtkWidget *widget,
 	/* appease GtkTreeView by preventing its drag_data_receive
 	 * from being called */
 	g_signal_stop_emission_by_name (dest->details->tree_view,
-					"drag_data_received");
+					"drag-data-received");
 
 	return TRUE;
 }
@@ -984,7 +1045,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 	g_type_class_add_private (class, sizeof (NautilusTreeViewDragDestDetails));
 
 	signals[GET_ROOT_URI] = 
-		g_signal_new ("get_root_uri",
+		g_signal_new ("get-root-uri",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass,
@@ -993,7 +1054,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 			      g_cclosure_marshal_generic,
 			      G_TYPE_STRING, 0);
 	signals[GET_FILE_FOR_PATH] = 
-		g_signal_new ("get_file_for_path",
+		g_signal_new ("get-file-for-path",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass,
@@ -1003,7 +1064,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 			      NAUTILUS_TYPE_FILE, 1,
 			      GTK_TYPE_TREE_PATH);
 	signals[MOVE_COPY_ITEMS] =
-		g_signal_new ("move_copy_items",
+		g_signal_new ("move-copy-items",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass,
@@ -1018,7 +1079,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 			      G_TYPE_INT,
 			      G_TYPE_INT);
 	signals[HANDLE_NETSCAPE_URL] =
-		g_signal_new ("handle_netscape_url",
+		g_signal_new ("handle-netscape-url",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass, 
@@ -1032,7 +1093,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 			      G_TYPE_INT,
 			      G_TYPE_INT);
 	signals[HANDLE_URI_LIST] =
-		g_signal_new ("handle_uri_list",
+		g_signal_new ("handle-uri-list",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass, 
@@ -1046,7 +1107,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 			      G_TYPE_INT,
 			      G_TYPE_INT);
 	signals[HANDLE_TEXT] =
-		g_signal_new ("handle_text",
+		g_signal_new ("handle-text",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass, 
@@ -1060,7 +1121,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 			      G_TYPE_INT,
 			      G_TYPE_INT);
 	signals[HANDLE_RAW] =
-		g_signal_new ("handle_raw",
+		g_signal_new ("handle-raw",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass,
@@ -1075,6 +1136,16 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 			      GDK_TYPE_DRAG_ACTION,
 			      G_TYPE_INT,
 			      G_TYPE_INT);
+	signals[HANDLE_HOVER] =
+		g_signal_new ("handle-hover",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass,
+					       handle_hover),
+			      NULL, NULL,
+			      g_cclosure_marshal_generic,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_STRING);
 }
 
 
@@ -1099,19 +1170,19 @@ nautilus_tree_view_drag_dest_new (GtkTreeView *tree_view)
 	gtk_target_list_add_text_targets (targets, NAUTILUS_ICON_DND_TEXT);
 
 	g_signal_connect_object (tree_view,
-				 "drag_motion",
+				 "drag-motion",
 				 G_CALLBACK (drag_motion_callback),
 				 dest, 0);
 	g_signal_connect_object (tree_view,
-				 "drag_leave",
+				 "drag-leave",
 				 G_CALLBACK (drag_leave_callback),
 				 dest, 0);
 	g_signal_connect_object (tree_view,
-				 "drag_drop",
+				 "drag-drop",
 				 G_CALLBACK (drag_drop_callback),
 				 dest, 0);
 	g_signal_connect_object (tree_view, 
-				 "drag_data_received",
+				 "drag-data-received",
 				 G_CALLBACK (drag_data_received_callback),
 				 dest, 0);
 	
