@@ -103,6 +103,7 @@ struct _NautilusApplicationPriv {
 	NautilusFreedesktopDBus *fdb_manager;
 
 	gboolean no_desktop;
+	gboolean force_desktop;
 	gchar *geometry;
 
 	NotifyNotification *unmount_notify;
@@ -262,7 +263,7 @@ menu_provider_items_updated_handler (NautilusMenuProvider *provider, GtkWidget*
 {
 
 	g_signal_emit_by_name (nautilus_signaller_get_current (),
-			       "popup_menu_changed");
+			       "popup-menu-changed");
 }
 
 static void
@@ -276,7 +277,7 @@ menu_provider_init_callback (void)
         for (l = providers; l != NULL; l = l->next) {
                 NautilusMenuProvider *provider = NAUTILUS_MENU_PROVIDER (l->data);
 
-		g_signal_connect_after (G_OBJECT (provider), "items_updated",
+		g_signal_connect_after (G_OBJECT (provider), "items-updated",
                            (GCallback)menu_provider_items_updated_handler,
                            NULL);
         }
@@ -429,7 +430,7 @@ get_desktop_manager_selection (GdkDisplay *display, int screen)
 						 selection_atom,
 						 gdk_x11_get_server_time (gtk_widget_get_window (selection_widget)))) {
 		
-		g_signal_connect (selection_widget, "selection_get",
+		g_signal_connect (selection_widget, "selection-get",
 				  G_CALLBACK (selection_get_cb), NULL);
 		return selection_widget;
 	}
@@ -479,7 +480,7 @@ nautilus_application_create_desktop_windows (NautilusApplication *application)
 			window = nautilus_desktop_window_new (GTK_APPLICATION (application),
 							      gdk_display_get_screen (display, i));
 
-			g_signal_connect (selection_widget, "selection_clear_event",
+			g_signal_connect (selection_widget, "selection-clear-event",
 					  G_CALLBACK (selection_clear_event_cb), window);
 			
 			g_signal_connect (window, "unrealize",
@@ -1042,6 +1043,12 @@ do_cmdline_sanity_checks (NautilusApplication *self,
 		goto out;
 	}
 
+	if (self->priv->no_desktop && self->priv->force_desktop) {
+		g_printerr ("%s\n",
+			    _("--no-desktop and --force-desktop cannot be used together."));
+		goto out;
+	}
+
 	retval = TRUE;
 
  out:
@@ -1148,7 +1155,9 @@ nautilus_application_local_command_line (GApplication *application,
 		{ "no-default-window", 'n', 0, G_OPTION_ARG_NONE, &no_default_window,
 		  N_("Only create windows for explicitly specified URIs."), NULL },
 		{ "no-desktop", '\0', 0, G_OPTION_ARG_NONE, &self->priv->no_desktop,
-		  N_("Do not manage the desktop (ignore the preference set in the preferences dialog)."), NULL },
+		  N_("Never manage the desktop (ignore the GSettings preference)."), NULL },
+		{ "force-desktop", '\0', 0, G_OPTION_ARG_NONE, &self->priv->force_desktop,
+		  N_("Always manage the desktop (ignore the GSettings preference)."), NULL },
 		{ "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell, 
 		  N_("Quit Nautilus."), NULL },
 		{ "select", 's', 0, G_OPTION_ARG_NONE, &select_uris,
@@ -1200,8 +1209,9 @@ nautilus_application_local_command_line (GApplication *application,
 	}
 
 	DEBUG ("Parsing local command line, no_default_window %d, quit %d, "
-	       "self checks %d, no_desktop %d",
-	       no_default_window, kill_shell, perform_self_check, self->priv->no_desktop);
+	       "self checks %d, no_desktop %d, show_desktop %d",
+	       no_default_window, kill_shell, perform_self_check,
+	       self->priv->no_desktop, self->priv->force_desktop);
 
 	g_application_register (application, NULL, &error);
 
@@ -1326,13 +1336,20 @@ desktop_changed_callback (gpointer user_data)
 static void
 init_desktop (NautilusApplication *self)
 {
-	if (!self->priv->no_desktop &&
-	    !g_settings_get_boolean (gnome_background_preferences,
-				     NAUTILUS_PREFERENCES_SHOW_DESKTOP)) {
-		self->priv->no_desktop = TRUE;
+	gboolean should_show;
+
+	/* by default, take the GSettings value */
+	should_show = g_settings_get_boolean (gnome_background_preferences,
+					      NAUTILUS_PREFERENCES_SHOW_DESKTOP);
+
+	/* the command line switches take over the setting */
+	if (self->priv->no_desktop) {
+		should_show = FALSE;
+	} else if (self->priv->force_desktop) {
+		should_show = TRUE;
 	}
 
-	if (!self->priv->no_desktop) {
+	if (should_show) {
 		nautilus_application_open_desktop (self);
 	}
 
