@@ -33,7 +33,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include <libegg/eggtreemultidnd.h>
 #include <eel/eel-graphic-effects.h>
 #include <libnautilus-private/nautilus-dnd.h>
 
@@ -56,7 +55,6 @@ static int nautilus_list_model_file_entry_compare_func (gconstpointer a,
 							gpointer      user_data);
 static void nautilus_list_model_tree_model_init (GtkTreeModelIface *iface);
 static void nautilus_list_model_sortable_init (GtkTreeSortableIface *iface);
-static void nautilus_list_model_multi_drag_source_init (EggTreeMultiDragSourceIface *iface);
 
 struct NautilusListModelDetails {
 	GSequence *files;
@@ -101,17 +99,13 @@ G_DEFINE_TYPE_WITH_CODE (NautilusListModel, nautilus_list_model, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
 						nautilus_list_model_tree_model_init)
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_SORTABLE,
-						nautilus_list_model_sortable_init)
-			 G_IMPLEMENT_INTERFACE (EGG_TYPE_TREE_MULTI_DRAG_SOURCE,
-						nautilus_list_model_multi_drag_source_init));
+						nautilus_list_model_sortable_init));
 
 static const GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
 };
 
-static GtkTargetList *drag_target_list = NULL;
-
 static void
 file_entry_free (FileEntry *file_entry)
 {
@@ -765,101 +759,6 @@ nautilus_list_model_has_default_sort_func (GtkTreeSortable *sortable)
 	return FALSE;
 }
 
-static gboolean
-nautilus_list_model_multi_row_draggable (EggTreeMultiDragSource *drag_source, GList *path_list)
-{
-	return TRUE;
-}
-
-static void
-each_path_get_data_binder (NautilusDragEachSelectedItemDataGet data_get,
-			   gpointer context,
-			   gpointer data)
-{
-	DragDataGetInfo *info;
-	GList *l;
-	NautilusFile *file;
-	GtkTreeRowReference *row;
-	GtkTreePath *path;
-	char *uri;
-	GdkRectangle cell_area;
-	GtkTreeViewColumn *column;
-
-	info = context;
-
-	g_return_if_fail (info->model->details->drag_view);
-
-	column = gtk_tree_view_get_column (info->model->details->drag_view, 0);
-
-	for (l = info->path_list; l != NULL; l = l->next) {
-		row = l->data;
-
-		path = gtk_tree_row_reference_get_path (row);
-		file = nautilus_list_model_file_for_path (info->model, path);
-		if (file) {
-			gtk_tree_view_get_cell_area
-				(info->model->details->drag_view,
-				 path, 
-				 column,
-				 &cell_area);
-				
-			uri = nautilus_file_get_uri (file);
-				
-			(*data_get) (uri, 
-				     0,
-				     cell_area.y - info->model->details->drag_begin_y,
-				     cell_area.width, cell_area.height, 
-				     data);
-				
-			g_free (uri);
-			
-			nautilus_file_unref (file);
-		}
-		
-		gtk_tree_path_free (path);
-	}
-}
-
-static gboolean
-nautilus_list_model_multi_drag_data_get (EggTreeMultiDragSource *drag_source, 
-					 GList *path_list, 
-					 GtkSelectionData *selection_data)
-{
-	NautilusListModel *model;
-	DragDataGetInfo context;
-	guint target_info;
-	
-	model = NAUTILUS_LIST_MODEL (drag_source);
-
-	context.model = model;
-	context.path_list = path_list;
-
-	if (!drag_target_list) {
-		drag_target_list = nautilus_list_model_get_drag_target_list ();
-	}
-
-	if (gtk_target_list_find (drag_target_list,
-				  gtk_selection_data_get_target (selection_data),
-				  &target_info)) {
-		nautilus_drag_drag_data_get (NULL,
-					     NULL,
-					     selection_data,
-					     target_info,
-					     GDK_CURRENT_TIME,
-					     &context,
-					     each_path_get_data_binder);
-		return TRUE;
-	} else {
-		return FALSE;
-	}
-}
-
-static gboolean
-nautilus_list_model_multi_drag_data_delete (EggTreeMultiDragSource *drag_source, GList *path_list)
-{
-	return TRUE;
-}
-
 static void
 add_dummy_row (NautilusListModel *model, FileEntry *parent_entry)
 {
@@ -1392,6 +1291,22 @@ nautilus_list_model_set_drag_view (NautilusListModel *model,
 	model->details->drag_begin_y = drag_begin_y;
 }
 
+GtkTreeView *
+nautilus_list_model_get_drag_view (NautilusListModel *model,
+				   int *drag_begin_x,
+				   int *drag_begin_y)
+{
+	if (drag_begin_x != NULL) {
+		*drag_begin_x = model->details->drag_begin_x;
+	}
+
+	if (drag_begin_y != NULL) {
+		*drag_begin_y = model->details->drag_begin_y;
+	}
+
+	return model->details->drag_view;
+}
+
 GtkTargetList *
 nautilus_list_model_get_drag_target_list ()
 {
@@ -1513,7 +1428,7 @@ nautilus_list_model_class_init (NautilusListModelClass *klass)
 	object_class->dispose = nautilus_list_model_dispose;
 
       list_model_signals[SUBDIRECTORY_UNLOADED] =
-        g_signal_new ("subdirectory_unloaded",
+        g_signal_new ("subdirectory-unloaded",
                       NAUTILUS_TYPE_LIST_MODEL,
                       G_SIGNAL_RUN_FIRST,
                       G_STRUCT_OFFSET (NautilusListModelClass, subdirectory_unloaded),
@@ -1548,14 +1463,6 @@ nautilus_list_model_sortable_init (GtkTreeSortableIface *iface)
 	iface->has_default_sort_func = nautilus_list_model_has_default_sort_func;
 }
 
-static void
-nautilus_list_model_multi_drag_source_init (EggTreeMultiDragSourceIface *iface)
-{
-	iface->row_draggable = nautilus_list_model_multi_row_draggable;
-	iface->drag_data_get = nautilus_list_model_multi_drag_data_get;
-	iface->drag_data_delete = nautilus_list_model_multi_drag_data_delete;
-}
-
 void
 nautilus_list_model_subdirectory_done_loading (NautilusListModel *model, NautilusDirectory *directory)
 {
