@@ -40,7 +40,6 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <libegg/eggtreemultidnd.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <libnautilus-extension/nautilus-column-provider.h>
@@ -354,7 +353,7 @@ drag_data_get_callback (GtkWidget *widget,
 {
 	GtkTreeView *tree_view;
 	GtkTreeModel *model;
-	GList *ref_list;
+	GList *selection_cache;
 
 	tree_view = GTK_TREE_VIEW (widget);
   
@@ -364,67 +363,12 @@ drag_data_get_callback (GtkWidget *widget,
 		return;
 	}
 
-	ref_list = g_object_get_data (G_OBJECT (context), "drag-info");
-
-	if (ref_list == NULL) {
+	selection_cache = g_object_get_data (G_OBJECT (context), "drag-info");
+	if (selection_cache == NULL) {
 		return;
 	}
 
-	if (EGG_IS_TREE_MULTI_DRAG_SOURCE (model)) {
-		egg_tree_multi_drag_source_drag_data_get (EGG_TREE_MULTI_DRAG_SOURCE (model),
-							  ref_list,
-							  selection_data);
-	}
-}
-
-static void
-filtered_selection_foreach (GtkTreeModel *model,
-			    GtkTreePath *path,
-			    GtkTreeIter *iter,
-			    gpointer data)
-{
-	struct SelectionForeachData *selection_data;
-	GtkTreeIter parent;
-	GtkTreeIter child;
-	
-	selection_data = data;
-
-	/* If the parent folder is also selected, don't include this file in the
-	 * file operation, since that would copy it to the toplevel target instead
-	 * of keeping it as a child of the copied folder
-	 */
-	child = *iter;
-	while (gtk_tree_model_iter_parent (model, &parent, &child)) {
-		if (gtk_tree_selection_iter_is_selected (selection_data->selection,
-							 &parent)) {
-			return;
-		}
-		child = parent;
-	}
-	
-	selection_data->list = g_list_prepend (selection_data->list, 
-					       gtk_tree_row_reference_new (model, path));
-}
-
-static GList *
-get_filtered_selection_refs (GtkTreeView *tree_view)
-{
-	struct SelectionForeachData selection_data;
-
-	selection_data.list = NULL;
-	selection_data.selection = gtk_tree_view_get_selection (tree_view);
-	
-	gtk_tree_selection_selected_foreach (selection_data.selection, 
-					     filtered_selection_foreach, 
-					     &selection_data);
-	return g_list_reverse (selection_data.list);
-}
-
-static void
-ref_list_free (GList *ref_list)
-{
-	g_list_foreach (ref_list, (GFunc) gtk_tree_row_reference_free, NULL);
-	g_list_free (ref_list);
+	nautilus_drag_drag_data_get_from_cache (selection_cache, context, selection_data, info, time);
 }
 
 static void
@@ -466,12 +410,80 @@ get_drag_pixbuf (NautilusListView *view)
 	return ret;
 }
 
+/* iteration glue struct */
+typedef struct {
+	NautilusListView *view;
+	NautilusDragEachSelectedItemDataGet iteratee;
+	gpointer iteratee_data;
+} ListGetDataBinderContext;
+
+static void
+item_get_data_binder (GtkTreeModel *model,
+		      GtkTreePath *path,
+		      GtkTreeIter *iter,
+		      gpointer data)
+{
+	ListGetDataBinderContext *context = data;
+	NautilusFile *file;
+	GtkTreeView *treeview;
+	GtkTreeViewColumn *column;
+	GdkRectangle cell_area;
+	int drag_begin_y  = 0;
+	char *uri;
+
+	treeview = nautilus_list_model_get_drag_view (context->view->details->model,
+						      NULL,
+						      &drag_begin_y);
+	column = gtk_tree_view_get_column (treeview, 0);
+
+	file = nautilus_list_model_file_for_path (NAUTILUS_LIST_MODEL (model), path);
+	if (file == NULL) {
+		return;
+	}
+
+	gtk_tree_view_get_cell_area (treeview,
+				     path,
+				     column,
+				     &cell_area);
+
+	uri = nautilus_file_get_uri (file);
+	nautilus_file_unref (file);
+
+	/* pass the uri, mouse-relative x/y and icon width/height */
+	context->iteratee (uri,
+			   0,
+			   cell_area.y - drag_begin_y,
+			   cell_area.width,
+			   cell_area.height,
+			   context->iteratee_data);
+
+	g_free (uri);
+}
+
+static void
+each_item_get_data_binder (NautilusDragEachSelectedItemDataGet iteratee,
+			   gpointer iterator_context,
+			   gpointer data)
+{
+	NautilusListView *view = NAUTILUS_LIST_VIEW (iterator_context);
+	ListGetDataBinderContext context;
+	GtkTreeSelection *selection;
+
+	context.view = view;
+	context.iteratee = iteratee;
+	context.iteratee_data = data;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->details->tree_view));
+	gtk_tree_selection_selected_foreach (selection, item_get_data_binder, &context);
+}
+
+
 static void
 drag_begin_callback (GtkWidget *widget,
 		     GdkDragContext *context,
 		     NautilusListView *view)
 {
-	GList *ref_list;
+	GList *selection_cache;
 	GdkPixbuf *pixbuf;
 
 	pixbuf = get_drag_pixbuf (view);
@@ -486,12 +498,14 @@ drag_begin_callback (GtkWidget *widget,
 
 	stop_drag_check (view);
 	view->details->drag_started = TRUE;
-	
-	ref_list = get_filtered_selection_refs (GTK_TREE_VIEW (widget));
+
+	selection_cache = nautilus_drag_create_selection_cache (view,
+								each_item_get_data_binder);
+
 	g_object_set_data_full (G_OBJECT (context),
 				"drag-info",
-				ref_list,
-				(GDestroyNotify)ref_list_free);
+				selection_cache,
+				(GDestroyNotify)nautilus_drag_destroy_selection_list);
 }
 
 static gboolean
@@ -1197,6 +1211,14 @@ list_view_handle_raw (NautilusTreeViewDragDest *dest, const char *raw_data,
 				       action, x, y);
 }
 
+static void
+list_view_handle_hover (NautilusTreeViewDragDest *dest,
+			const char *target_uri,
+			NautilusListView *view)
+{
+	nautilus_view_handle_hover (NAUTILUS_VIEW (view), target_uri);
+}
+
 static void
 move_copy_items_callback (NautilusTreeViewDragDest *dest,
 			  const GList *item_uris,
@@ -1381,47 +1403,49 @@ create_and_set_up_tree_view (NautilusListView *view)
 		nautilus_tree_view_drag_dest_new (view->details->tree_view);
 
 	g_signal_connect_object (view->details->drag_dest,
-				 "get_root_uri",
+				 "get-root-uri",
 				 G_CALLBACK (get_root_uri_callback),
 				 view, 0);
 	g_signal_connect_object (view->details->drag_dest,
-				 "get_file_for_path",
+				 "get-file-for-path",
 				 G_CALLBACK (get_file_for_path_callback),
 				 view, 0);
 	g_signal_connect_object (view->details->drag_dest,
-				 "move_copy_items",
+				 "move-copy-items",
 				 G_CALLBACK (move_copy_items_callback),
 				 view, 0);
-	g_signal_connect_object (view->details->drag_dest, "handle_netscape_url",
+	g_signal_connect_object (view->details->drag_dest, "handle-netscape-url",
 				 G_CALLBACK (list_view_handle_netscape_url), view, 0);
-	g_signal_connect_object (view->details->drag_dest, "handle_uri_list",
+	g_signal_connect_object (view->details->drag_dest, "handle-uri-list",
 				 G_CALLBACK (list_view_handle_uri_list), view, 0);
-	g_signal_connect_object (view->details->drag_dest, "handle_text",
+	g_signal_connect_object (view->details->drag_dest, "handle-text",
 				 G_CALLBACK (list_view_handle_text), view, 0);
-	g_signal_connect_object (view->details->drag_dest, "handle_raw",
+	g_signal_connect_object (view->details->drag_dest, "handle-raw",
 				 G_CALLBACK (list_view_handle_raw), view, 0);
+	g_signal_connect_object (view->details->drag_dest, "handle-hover",
+				 G_CALLBACK (list_view_handle_hover), view, 0);
 
 	g_signal_connect_object (gtk_tree_view_get_selection (view->details->tree_view),
 				 "changed",
 				 G_CALLBACK (list_selection_changed_callback), view, 0);
 
-	g_signal_connect_object (view->details->tree_view, "drag_begin",
+	g_signal_connect_object (view->details->tree_view, "drag-begin",
 				 G_CALLBACK (drag_begin_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "drag_data_get",
+	g_signal_connect_object (view->details->tree_view, "drag-data-get",
 				 G_CALLBACK (drag_data_get_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "motion_notify_event",
+	g_signal_connect_object (view->details->tree_view, "motion-notify-event",
 				 G_CALLBACK (motion_notify_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "enter_notify_event",
+	g_signal_connect_object (view->details->tree_view, "enter-notify-event",
 				 G_CALLBACK (enter_notify_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "leave_notify_event",
+	g_signal_connect_object (view->details->tree_view, "leave-notify-event",
 				 G_CALLBACK (leave_notify_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "button_press_event",
+	g_signal_connect_object (view->details->tree_view, "button-press-event",
 				 G_CALLBACK (button_press_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "button_release_event",
+	g_signal_connect_object (view->details->tree_view, "button-release-event",
 				 G_CALLBACK (button_release_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "key_press_event",
+	g_signal_connect_object (view->details->tree_view, "key-press-event",
 				 G_CALLBACK (key_press_callback), view, 0);
-	g_signal_connect_object (view->details->tree_view, "popup_menu",
+	g_signal_connect_object (view->details->tree_view, "popup-menu",
                                  G_CALLBACK (popup_menu_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "row-activated",
                                  G_CALLBACK (row_activated_callback), view, 0);
@@ -1432,10 +1456,10 @@ create_and_set_up_tree_view (NautilusListView *view)
 	nautilus_list_model_set_drag_view (NAUTILUS_LIST_MODEL (view->details->model),
 				     view->details->tree_view,  0, 0);
 
-	g_signal_connect_object (view->details->model, "sort_column_changed",
+	g_signal_connect_object (view->details->model, "sort-column-changed",
 				 G_CALLBACK (sort_column_changed_callback), view, 0);
 	
-	g_signal_connect_object (view->details->model, "subdirectory_unloaded",
+	g_signal_connect_object (view->details->model, "subdirectory-unloaded",
 				 G_CALLBACK (subdirectory_unloaded_callback), view, 0);
 
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (view->details->tree_view), GTK_SELECTION_MULTIPLE);
@@ -2294,7 +2318,7 @@ create_column_editor (NautilusListView *view)
 	g_signal_connect (column_chooser, "changed",
 			  G_CALLBACK (column_chooser_changed_callback),
 			  view);
-	g_signal_connect (column_chooser, "use_default",
+	g_signal_connect (column_chooser, "use-default",
 			  G_CALLBACK (column_chooser_use_default_callback),
 			  view);
 
@@ -2432,13 +2456,13 @@ nautilus_list_view_set_zoom_level (NautilusListView *view,
 
 	if (view->details->zoom_level == new_level) {
 		if (always_emit) {
-			g_signal_emit_by_name (NAUTILUS_VIEW(view), "zoom_level_changed");
+			g_signal_emit_by_name (NAUTILUS_VIEW(view), "zoom-level-changed");
 		}
 		return;
 	}
 
 	view->details->zoom_level = new_level;
-	g_signal_emit_by_name (NAUTILUS_VIEW(view), "zoom_level_changed");
+	g_signal_emit_by_name (NAUTILUS_VIEW(view), "zoom-level-changed");
 
 	/* Select correctly scaled icons. */
 	column = nautilus_list_model_get_column_id_from_zoom_level (new_level);
@@ -2956,7 +2980,7 @@ nautilus_list_view_init (NautilusListView *list_view)
 	list_view->details->hover_path = NULL;
 	list_view->details->clipboard_handler_id =
 		g_signal_connect (nautilus_clipboard_monitor_get (),
-		                  "clipboard_info",
+		                  "clipboard-info",
 		                  G_CALLBACK (list_view_notify_clipboard_info), list_view);
 }
 
