@@ -76,6 +76,7 @@ typedef struct {
 	gboolean  drag_data_received;
 	int       drag_data_info;
 	gboolean  drop_occured;
+	char     *target_uri;
 
 	GtkWidget *popup_menu;
 	GtkWidget *popup_menu_open_in_new_tab_item;
@@ -99,9 +100,11 @@ typedef struct {
 	NautilusWindowOpenFlags go_to_after_mount_flags;
 
 	GDBusProxy *hostnamed_proxy;
+	GCancellable *hostnamed_cancellable;
 	char *hostname;
 
 	guint bookmarks_changed_id;
+	guint switch_location_timer;
 } NautilusPlacesSidebar;
 
 typedef struct {
@@ -379,7 +382,8 @@ add_special_dirs (NautilusPlacesSidebar *sidebar)
 
 	dirs = NULL;
 	for (index = 0; index < G_USER_N_DIRECTORIES; index++) {
-		const char *path, *name;
+		const char *path;
+		char *name;
 		GFile *root;
 		GIcon *icon;
 		char *mount_uri;
@@ -434,6 +438,7 @@ add_special_dirs (NautilusPlacesSidebar *sidebar)
 
 		g_object_unref (root);
 		g_object_unref (icon);
+		g_free (name);
 		g_free (mount_uri);
 
 		dirs = g_list_prepend (dirs, (char *)path);
@@ -1122,6 +1127,15 @@ get_drag_data (GtkTreeView *tree_view,
 	return TRUE;
 }
 
+static void
+remove_switch_location_timer (NautilusPlacesSidebar *sidebar)
+{
+	if (sidebar->switch_location_timer != 0) {
+		g_source_remove (sidebar->switch_location_timer);
+		sidebar->switch_location_timer = 0;
+	}
+}
+
 static void
 free_drag_data (NautilusPlacesSidebar *sidebar)
 {
@@ -1131,6 +1145,56 @@ free_drag_data (NautilusPlacesSidebar *sidebar)
 		nautilus_drag_destroy_selection_list (sidebar->drag_list);
 		sidebar->drag_list = NULL;
 	}
+
+	remove_switch_location_timer (sidebar);
+
+	g_free (sidebar->target_uri);
+	sidebar->target_uri = NULL;
+}
+
+static gboolean
+switch_location_timer (gpointer user_data)
+{
+	NautilusPlacesSidebar *sidebar = user_data;
+	GFile *location;
+	NautilusWindowSlot *target_slot;
+
+	sidebar->switch_location_timer = 0;
+
+	target_slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (sidebar->window));
+
+	location = g_file_new_for_uri (sidebar->target_uri);
+	nautilus_window_slot_open_location (target_slot, location, 0);
+	g_object_unref (location);
+
+	return FALSE;
+}
+
+static void
+check_switch_location_timer (NautilusPlacesSidebar *sidebar,
+			     const char            *uri)
+{
+	GtkSettings *settings;
+	guint timeout;
+
+	if (g_strcmp0 (uri, sidebar->target_uri) == 0) {
+		return;
+	}
+	remove_switch_location_timer (sidebar);
+
+	settings = gtk_widget_get_settings (GTK_WIDGET (sidebar));
+	g_object_get (settings, "gtk-timeout-expand", &timeout, NULL);
+
+	g_free (sidebar->target_uri);
+	sidebar->target_uri = NULL;
+
+	if (uri != NULL) {
+		sidebar->target_uri = g_strdup (uri);
+		sidebar->switch_location_timer =
+			gdk_threads_add_timeout (timeout,
+						 switch_location_timer,
+						 sidebar);
+	}
 }
 
 static gboolean
@@ -1145,7 +1209,7 @@ drag_motion_callback (GtkTreeView *tree_view,
 	GtkTreeViewDropPosition pos;
 	int action;
 	GtkTreeIter iter;
-	char *uri;
+	char *target_uri = NULL;
 	gboolean res;
 
 	if (!sidebar->drag_data_received) {
@@ -1176,12 +1240,11 @@ drag_motion_callback (GtkTreeView *tree_view,
 						 &iter, path);
 			gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store),
 					    &iter,
-					    PLACES_SIDEBAR_COLUMN_URI, &uri,
+					    PLACES_SIDEBAR_COLUMN_URI, &target_uri,
 					    -1);
-			nautilus_drag_default_drop_action_for_icons (context, uri,
+			nautilus_drag_default_drop_action_for_icons (context, target_uri,
 								     sidebar->drag_list,
 								     &action);
-			g_free (uri);
 		}
 	}
 
@@ -1197,11 +1260,15 @@ drag_motion_callback (GtkTreeView *tree_view,
 	g_signal_stop_emission_by_name (tree_view, "drag-motion");
 
 	if (action != 0) {
+		check_switch_location_timer (sidebar, target_uri);
 		gdk_drag_status (context, action, time);
 	} else {
+		remove_switch_location_timer (sidebar);
 		gdk_drag_status (context, 0, time);
 	}
 
+	g_free (target_uri);
+
 	return TRUE;
 }
 
@@ -1242,12 +1309,12 @@ build_selection_list (const char *data)
 	int i;
 
 	uris = g_uri_list_extract_uris (data);
-
 	result = NULL;
 	for (i = 0; uris[i]; i++) {
 		uri = uris[i];
 		item = nautilus_drag_selection_item_new ();
 		item->uri = g_strdup (uri);
+		item->file = nautilus_file_get_existing_by_uri (uri);
 		item->got_icon_position = FALSE;
 		result = g_list_prepend (result, item);
 	}
@@ -1400,7 +1467,9 @@ drag_data_received_callback (GtkWidget *widget,
 			switch (info) {
 			case TEXT_URI_LIST:
 				selection_list = build_selection_list ((const gchar *) gtk_selection_data_get_data (selection_data));
+				g_assert (selection_list);
 				uris = uri_list_from_selection (selection_list);
+				g_assert (uris);
 				nautilus_file_operations_copy_move (uris, NULL, drop_uri,
 								    real_action, GTK_WIDGET (tree_view),
 								    NULL, NULL);
@@ -2738,6 +2807,22 @@ bookmarks_popup_menu_cb (GtkWidget *widget,
 	return TRUE;
 }
 
+static void
+bookmarks_row_activated_cb (GtkWidget *widget,
+			    GtkTreePath *path,
+			    GtkTreeViewColumn *column,
+			    NautilusPlacesSidebar *sidebar)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
+
+	if (!gtk_tree_model_get_iter (model, &iter, path)) {
+		return;
+	}
+
+	open_selected_bookmark (sidebar, model, &iter, 0);
+}
+
 static gboolean
 bookmarks_button_release_event_cb (GtkWidget *widget,
 				   GdkEventButton *event,
@@ -2763,6 +2848,10 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 		return FALSE;
 	}
 
+	if (event->button == 1) {
+		return FALSE;
+	}
+
 	tree_view = GTK_TREE_VIEW (widget);
 	model = gtk_tree_view_get_model (tree_view);
 
@@ -2783,9 +2872,7 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 		return FALSE;
 	}
 
-	if (event->button == 1) {
-		open_selected_bookmark (sidebar, model, &iter, 0);
-	} else if (event->button == 2) {
+	if (event->button == 2) {
 		NautilusWindowOpenFlags flags = 0;
 
 		flags = (event->state & GDK_CONTROL_MASK) ?
@@ -3061,6 +3148,8 @@ hostname_proxy_new_cb (GObject      *source_object,
 	GError *error = NULL;
 
 	sidebar->hostnamed_proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
+	g_clear_object (&sidebar->hostnamed_cancellable);
+
 	if (error != NULL) {
 		g_debug ("Failed to create D-Bus proxy: %s", error->message);
 		g_error_free (error);
@@ -3252,27 +3341,29 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 			  G_CALLBACK (bookmarks_popup_menu_cb), sidebar);
 	g_signal_connect (tree_view, "button-release-event",
 			  G_CALLBACK (bookmarks_button_release_event_cb), sidebar);
+	g_signal_connect (tree_view, "row-activated",
+			  G_CALLBACK (bookmarks_row_activated_cb), sidebar);
 
-	eel_gtk_tree_view_set_activate_on_single_click (sidebar->tree_view,
-							TRUE);
+	gtk_tree_view_set_activate_on_single_click (sidebar->tree_view, TRUE);
 
 	g_signal_connect_swapped (gnome_background_preferences, "changed::" NAUTILUS_PREFERENCES_SHOW_DESKTOP,
 				  G_CALLBACK(desktop_setting_changed_callback),
 				  sidebar);
 
 	sidebar->hostname = g_strdup (_("Computer"));
+	sidebar->hostnamed_cancellable = g_cancellable_new ();
 	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
 				  G_DBUS_PROXY_FLAGS_GET_INVALIDATED_PROPERTIES,
 				  NULL,
 				  "org.freedesktop.hostname1",
 				  "/org/freedesktop/hostname1",
 				  "org.freedesktop.hostname1",
-				  NULL,
+				  sidebar->hostnamed_cancellable,
 				  hostname_proxy_new_cb,
 				  sidebar);
 
 	g_signal_connect_object (nautilus_trash_monitor_get (),
-				 "trash_state_changed",
+				 "trash-state-changed",
 				 G_CALLBACK (trash_state_changed_cb),
 				 sidebar, 0);
 }
@@ -3337,6 +3428,11 @@ nautilus_places_sidebar_dispose (GObject *object)
 		g_clear_object (&sidebar->volume_monitor);
 	}
 
+	if (sidebar->hostnamed_cancellable != NULL) {
+		g_cancellable_cancel (sidebar->hostnamed_cancellable);
+		g_clear_object (&sidebar->hostnamed_cancellable);
+	}
+
 	g_clear_object (&sidebar->hostnamed_proxy);
 	g_free (sidebar->hostname);
 	sidebar->hostname = NULL;
@@ -3370,23 +3466,23 @@ nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
 					  G_CALLBACK (update_places),
 					  sidebar);
 
-	g_signal_connect_object (sidebar->volume_monitor, "volume_added",
+	g_signal_connect_object (sidebar->volume_monitor, "volume-added",
 				 G_CALLBACK (volume_added_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "volume_removed",
+	g_signal_connect_object (sidebar->volume_monitor, "volume-removed",
 				 G_CALLBACK (volume_removed_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "volume_changed",
+	g_signal_connect_object (sidebar->volume_monitor, "volume-changed",
 				 G_CALLBACK (volume_changed_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "mount_added",
+	g_signal_connect_object (sidebar->volume_monitor, "mount-added",
 				 G_CALLBACK (mount_added_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "mount_removed",
+	g_signal_connect_object (sidebar->volume_monitor, "mount-removed",
 				 G_CALLBACK (mount_removed_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "mount_changed",
+	g_signal_connect_object (sidebar->volume_monitor, "mount-changed",
 				 G_CALLBACK (mount_changed_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "drive_disconnected",
+	g_signal_connect_object (sidebar->volume_monitor, "drive-disconnected",
 				 G_CALLBACK (drive_disconnected_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "drive_connected",
+	g_signal_connect_object (sidebar->volume_monitor, "drive-connected",
 				 G_CALLBACK (drive_connected_callback), sidebar, 0);
-	g_signal_connect_object (sidebar->volume_monitor, "drive_changed",
+	g_signal_connect_object (sidebar->volume_monitor, "drive-changed",
 				 G_CALLBACK (drive_changed_callback), sidebar, 0);
 
 	update_places (sidebar);
